@@ -6,16 +6,14 @@ A few popular time series datasets
 """
 
 from pathlib import Path
-from typing import List
 
 import numpy as np
 import pandas as pd
 
 from darts import TimeSeries
+from darts.datasets.dataset_loaders import DatasetLoaderCSV, DatasetLoaderMetadata
 from darts.logging import get_logger, raise_if_not
-from darts.utils.utils import _build_tqdm_iterator
-
-from .dataset_loaders import DatasetLoaderCSV, DatasetLoaderMetadata
+from darts.utils.utils import _build_tqdm_iterator, freqs
 
 """
     Overall usage of this package:
@@ -490,6 +488,32 @@ class ETTm2Dataset(DatasetLoaderCSV):
         )
 
 
+class TaxiNewYorkDataset(DatasetLoaderCSV):
+    """
+    Taxi Passengers in New York, from 2014-07 to 2015-01.
+    The data consists of aggregated total number of
+    taxi passengers into 30 minute buckets.
+    Univariate series.
+    Source: [1]_
+
+    References
+    ----------
+    .. [1] https://www.kaggle.com/code/julienjta/nyc-taxi-traffic-analysis
+    """
+
+    def __init__(self):
+        super().__init__(
+            metadata=DatasetLoaderMetadata(
+                "taxi_new_york_passengers.csv",
+                uri=_DEFAULT_PATH + "/taxi_new_york_passengers.csv",
+                hash="0a81adf1b74354a8ec18c30e9e8fe5f0",
+                header_time="time",
+                format_time="%Y-%m-%d %H:%M:%S",
+                freq="30min",
+            ),
+        )
+
+
 class ElectricityDataset(DatasetLoaderCSV):
     """
     Measurements of electric power consumption in one household with 15 minute sampling rate.
@@ -512,19 +536,20 @@ class ElectricityDataset(DatasetLoaderCSV):
         Parameters
         ----------
         multivariate: bool
-            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries. Default is True.
+            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries.
+            Default is True.
         """
 
         def pre_proces_fn(extracted_dir, dataset_path):
             with open(Path(extracted_dir, "LD2011_2014.txt")) as fin:
-                with open(dataset_path, "wt", newline="\n") as fout:
+                with open(dataset_path, "w", newline="\n") as fout:
                     for line in fin:
                         fout.write(line.replace(",", ".").replace(";", ","))
 
         super().__init__(
             metadata=DatasetLoaderMetadata(
                 "Electricity.csv",
-                uri="https://archive.ics.uci.edu/ml/machine-learning-databases/00321/LD2011_2014.txt.zip",
+                uri="https://archive.ics.uci.edu/static/public/321/electricityloaddiagrams20112014.zip",
                 hash="acfe6783eea43905e510f537add940fd",
                 header_time="Unnamed: 0",
                 format_time="%Y-%m-%d %H:%M:%S",
@@ -533,7 +558,7 @@ class ElectricityDataset(DatasetLoaderCSV):
             )
         )
 
-    def _to_multi_series(self, series: pd.DataFrame) -> List[TimeSeries]:
+    def _to_multi_series(self, series: pd.DataFrame) -> list[TimeSeries]:
         """
         Load the electricity dataset as a list of univariate series, one for each household.
         """
@@ -546,8 +571,8 @@ class ElectricityDataset(DatasetLoaderCSV):
 
             # filter column down to the period of recording
             srs = srs.replace(0.0, np.nan)
-            start_date = min(srs.fillna(method="ffill").dropna().index)
-            end_date = max(srs.fillna(method="bfill").dropna().index)
+            start_date = min(srs.ffill().dropna().index)
+            end_date = max(srs.bfill().dropna().index)
             active_range = (srs.index >= start_date) & (srs.index <= end_date)
             srs = srs[active_range].fillna(0.0)
 
@@ -583,7 +608,8 @@ class UberTLCDataset(DatasetLoaderCSV):
         sample_freq: str
             The sampling frequency of the data. Can be "hourly" or "daily". Default is "hourly".
         multivariate: bool
-            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries. Default is True.
+            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries.
+            Default is True.
         """
         valid_sample_freq = ["daily", "hourly"]
         raise_if_not(
@@ -601,7 +627,7 @@ class UberTLCDataset(DatasetLoaderCSV):
             )
 
             output_dict = {}
-            freq_setting = "1H" if "hourly" in str(dataset_path) else "1D"
+            freq_setting = "1" + freqs["h"] if "hourly" in str(dataset_path) else "1D"
             time_series_of_locations = list(df.groupby(by="locationID"))
             for locationID, df in time_series_of_locations:
                 df.sort_index()
@@ -611,7 +637,7 @@ class UberTLCDataset(DatasetLoaderCSV):
 
                 output_dict[locationID] = count_series
             output_df = pd.DataFrame(output_dict)
-            output_df.to_csv(dataset_path, line_terminator="\n")
+            output_df.to_csv(dataset_path)
 
         super().__init__(
             metadata=DatasetLoaderMetadata(
@@ -619,17 +645,19 @@ class UberTLCDataset(DatasetLoaderCSV):
                 uri="https://github.com/fivethirtyeight/uber-tlc-foil-response/raw/"
                 "63bb878b76f47f69b4527d50af57aac26dead983/"
                 "uber-trip-data/uber-raw-data-janjune-15.csv.zip",
-                hash="9ed84ebe0df4bc664748724b633b3fe6"
-                if sample_freq == "hourly"
-                else "24f9fd67e4b9e53f0214a90268cd9bee",
+                hash=(
+                    "9ed84ebe0df4bc664748724b633b3fe6"
+                    if sample_freq == "hourly"
+                    else "24f9fd67e4b9e53f0214a90268cd9bee"
+                ),
                 header_time="Pickup_date",
-                format_time="%Y-%m-%d %H:%M",
+                format_time="%Y-%m-%d %H:%M:%S",
                 pre_process_zipped_csv_fn=pre_proces_fn,
                 multivariate=multivariate,
             )
         )
 
-    def _to_multi_series(self, series: pd.DataFrame) -> List[TimeSeries]:
+    def _to_multi_series(self, series: pd.DataFrame) -> list[TimeSeries]:
         """
         load the Uber TLC dataset as a list of univariate timeseries, one for each locationID.
         """
@@ -641,8 +669,8 @@ class UberTLCDataset(DatasetLoaderCSV):
             srs = series[label]
 
             # filter column down to the period of recording
-            start_date = min(srs.fillna(method="ffill").dropna().index)
-            end_date = max(srs.fillna(method="bfill").dropna().index)
+            start_date = min(srs.ffill().dropna().index)
+            end_date = max(srs.bfill().dropna().index)
             active_range = (srs.index >= start_date) & (srs.index <= end_date)
             srs = srs[active_range]
 
@@ -662,15 +690,18 @@ class ILINetDataset(DatasetLoaderCSV):
 
     Components Descriptions:
 
-    * % WEIGHTED ILI: Combined state-specific data of patients visit to healthcare providers for ILI reported each week weighted by state population
-    * % UNWEIGHTED ILI: Combined state-specific data of patients visit to healthcare providers for ILI reported each week unweighted by state population
+    * % WEIGHTED ILI: Combined state-specific data of patients visit to healthcare providers for ILI reported each week
+        weighted by state population
+    * % UNWEIGHTED ILI: Combined state-specific data of patients visit to healthcare providers for ILI reported each
+        week unweighted by state population
     * AGE 0-4: Number of patients between 0 and 4 years of age
     * AGE 25-49: Number of patients between 25 and 49 years of age
     * AGE 25-64: Number of patients between 25 and 64 years of age
     * AGE 5-24: Number of patients between 5 and 24 years of age
     * AGE 50-64: Number of patients between 50 and 64 years of age
     * AGE 65: Number of patients above (>=65) 65 years of age
-    * ILITOTAL: Total number of ILI patients. For this system, ILI is defined as fever (temperature of 100°F [37.8°C] or greater) and a cough and/or a sore throat
+    * ILITOTAL: Total number of ILI patients. For this system, ILI is defined as fever (temperature of 100°F [37.8°C]
+        or greater) and a cough and/or a sore throat
     * NUM. OF PROVIDERS: Number of outpatient healthcare providers
     * TOTAL PATIENTS: Total number of patients
 
@@ -697,7 +728,7 @@ class ILINetDataset(DatasetLoaderCSV):
             )
         )
 
-    def _to_multi_series(self, series: pd.DataFrame) -> List[TimeSeries]:
+    def _to_multi_series(self, series: pd.DataFrame) -> list[TimeSeries]:
         """
         Load the ILINetDataset dataset as a list of univariate timeseries.
         """
@@ -706,8 +737,9 @@ class ILINetDataset(DatasetLoaderCSV):
 
 class ExchangeRateDataset(DatasetLoaderCSV):
     """
-    The collection of the daily exchange rates of eight foreign countries, including Australia, British, Canada, Switzerland, China, Japan, New Zealand,
-    and Singapore, ranging from 1990 to 2016. Unfortunately, there were some inconsistencies concerning the dates, so the resulting TimeSeries is integer-indexed.
+    The collection of the daily exchange rates of eight foreign countries, including Australia, British, Canada,
+    Switzerland, China, Japan, New Zealand, and Singapore, ranging from 1990 to 2016. Unfortunately,
+    there were some inconsistencies concerning the dates, so the resulting TimeSeries is integer-indexed.
     Source: [1]_
 
     References
@@ -720,7 +752,8 @@ class ExchangeRateDataset(DatasetLoaderCSV):
         Parameters
         ----------
         multivariate: bool
-            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries. Default is True.
+            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries.
+            Default is True.
         """
         super().__init__(
             metadata=DatasetLoaderMetadata(
@@ -732,7 +765,7 @@ class ExchangeRateDataset(DatasetLoaderCSV):
             )
         )
 
-    def _to_multi_series(self, series: pd.DataFrame) -> List[TimeSeries]:
+    def _to_multi_series(self, series: pd.DataFrame) -> list[TimeSeries]:
         """
         Load the ExchangeRateDataset dataset as a list of univariate timeseries, one for each country.
         """
@@ -741,8 +774,9 @@ class ExchangeRateDataset(DatasetLoaderCSV):
 
 class TrafficDataset(DatasetLoaderCSV):
     """
-    The data in this repo is a collection of 48 months (2015-2016) hourly data from the California Department of Transportation. The data describes
-    the road occupancy rates (between 0 and 1) measured by 862 different sensors on San Francisco Bay area freeways. The raw data is in http://pems.dot.ca.gov.
+    The data in this repo is a collection of 48 months (2015-2016) hourly data from the California Department
+    of Transportation. The data describes the road occupancy rates (between 0 and 1) measured by 862 different sensors
+    on San Francisco Bay area freeways. The raw data is in http://pems.dot.ca.gov.
     Source: [1]_
 
     References
@@ -755,7 +789,8 @@ class TrafficDataset(DatasetLoaderCSV):
         Parameters
         ----------
         multivariate: bool
-            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries. Default is True.
+            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries.
+            Default is True.
         """
         super().__init__(
             metadata=DatasetLoaderMetadata(
@@ -764,12 +799,12 @@ class TrafficDataset(DatasetLoaderCSV):
                 hash="a2105f364ef70aec06c757304833f72a",
                 header_time="Date",
                 format_time="%Y-%m-%d %H:%M:%S",
-                freq="1H",
+                freq="1" + freqs["h"],
                 multivariate=multivariate,
             )
         )
 
-    def _to_multi_series(self, series: pd.DataFrame) -> List[TimeSeries]:
+    def _to_multi_series(self, series: pd.DataFrame) -> list[TimeSeries]:
         """
         Load the TrafficDataset dataset as a list of univariate timeseries, one for each ID.
         """
@@ -794,7 +829,8 @@ class WeatherDataset(DatasetLoaderCSV):
         Parameters
         ----------
         multivariate: bool
-            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries. Default is True.
+            Whether to return a single multivariate timeseries - if False returns a list of univariate TimeSeries.
+            Default is True.
         """
         super().__init__(
             metadata=DatasetLoaderMetadata(
@@ -808,8 +844,119 @@ class WeatherDataset(DatasetLoaderCSV):
             )
         )
 
-    def _to_multi_series(self, series: pd.DataFrame) -> List[TimeSeries]:
+    def _to_multi_series(self, series: pd.DataFrame) -> list[TimeSeries]:
         """
         Load the WeatherDataset dataset as a list of univariate timeseries, one for weather indicator.
         """
         return [TimeSeries.from_series(series[label]) for label in series]
+
+
+class ElectricityConsumptionZurichDataset(DatasetLoaderCSV):
+    """
+    Electricity Consumption of households & SMEs (low voltage) and businesses & services (medium voltage) in the
+    city of Zurich [1]_, with values recorded every 15 minutes.
+
+    The electricity consumption is combined with weather measurements recorded by three different
+    stations in the city of Zurich with a hourly frequency [2]_. The missing time stamps are filled with NaN.
+    The original weather data is recorded every hour. Before adding the features to the electricity consumption,
+    the data is resampled to 15 minutes frequency, and missing values are interpolated.
+
+    To simplify the dataset, the measurements from the Zch_Schimmelstrasse and Zch_Rosengartenstrasse weather
+    stations are discarded to keep only the data recorded in the Zch_Stampfenbachstrasse station.
+
+    Both dataset sources are updated continuously, but this dataset only retrains values between 2015-01-01 and
+    2022-08-31.
+    The time index was converted from CET time zone to UTC.
+
+    Components Descriptions:
+
+    * Value_NE5 : Households & SMEs electricity consumption (low voltage, grid level 7) in kWh
+    * Value_NE7 : Business and services electricity consumption (medium voltage, grid level 5) in kWh
+    * Hr [%Hr] : Relative humidity
+    * RainDur [min] : Duration of precipitation (divided by 4 for conversion from hourly to quarter-hourly records)
+    * T [°C] : Temperature
+    * WD [°] : Wind direction
+    * WVv [m/s] : Wind vector speed
+    * p [hPa] : Air pressure
+    * WVs [m/s] : Wind scalar speed
+    * StrGlo [W/m2] : Global solar irradiation
+
+    Note: before 2018, the scalar speeds were calculated from the 30 minutes vector data.
+
+    The code below can be used to retrieve the dataset from the source. This might yield a dataset with differences
+    compared to the one stored in the repository, due to the fact that the source is updated continuously.
+
+    .. highlight:: python
+    .. code-block:: python
+
+        import pandas as pd
+
+        dataset_path = (
+            "https://data.stadt-zuerich.ch/dataset/"
+            "ewz_stromabgabe_netzebenen_stadt_zuerich/"
+            "download/ewz_stromabgabe_netzebenen_stadt_zuerich.csv"
+        )
+        df = pd.read_csv(dataset_path, index_col=0)
+        # convert time index
+        df.index = pd.DatetimeIndex(pd.to_datetime(df.index, utc=True)).tz_localize(
+            None
+        )
+        # extract pre-determined period
+        df = df.loc[
+            (pd.Timestamp("2015-01-01") <= df.index)
+            & (df.index <= pd.Timestamp("2022-08-31"))
+            ]
+        # download and preprocess the weather information
+        base_url = "https://data.stadt-zuerich.ch/dataset/ugz_meteodaten_stundenmittelwerte/download/"
+        filenames = [f"ugz_ogd_meteo_h1_{year}.csv" for year in range(2015, 2023)]
+        df_weather = pd.concat([pd.read_csv(base_url + fname) for fname in filenames])
+        # retain only one weather station
+        df_weather = df_weather.loc[df_weather["Standort"] == "Zch_Stampfenbachstrasse"]
+        # pivot the df to get all measurements as columns
+        df_weather["param_name"] = df_weather["Parameter"] + " [" + df_weather["Einheit"] + "]"
+        df_weather = df_weather.pivot(index="Datum", columns="param_name", values="Wert")
+        # convert time index to from CET to UTC and extract the required time range
+        df_weather.index = pd.DatetimeIndex(pd.to_datetime(df_weather.index, utc=True)).tz_localize(
+            None
+        )
+        df_weather = df_weather.loc[
+            (pd.Timestamp("2015-01-01") <= df_weather.index)
+            & (df_weather.index <= pd.Timestamp("2022-08-31"))
+        ]
+
+
+        # add weather data as additional features
+        df = pd.concat([df, df_weather], axis=1)
+        # interpolate weather data
+        df = df.interpolate()
+        # raining duration is given in minutes -> we divide by 4 from hourly to quarter-hourly records
+        df["RainDur [min]"] = df["RainDur [min]"] / 4
+
+        # round Electricity cols to 4 decimals, other columns to 2 decimals
+        cols_precise = ["Value_NE5", "Value_NE7"]
+        df = df.round(
+            decimals={col: (4 if col in cols_precise else 2) for col in df.columns}
+        )
+
+        # export the dataset
+        df.index.name = "Timestamp"
+        df.to_csv("electricity_consumption_zurich.csv")
+    ..
+
+
+    References
+    ----------
+    .. [1] https://data.stadt-zuerich.ch/dataset/ewz_stromabgabe_netzebenen_stadt_zuerich
+    .. [2] https://data.stadt-zuerich.ch/dataset/ugz_meteodaten_stundenmittelwerte
+    """
+
+    def __init__(self):
+        super().__init__(
+            metadata=DatasetLoaderMetadata(
+                "zurich_electricity_consumption.csv",
+                uri=_DEFAULT_PATH + "/zurich_electricity_consumption.csv",
+                hash="764bdd3bb4f9066ba7523accf9089505",
+                header_time="Timestamp",
+                freq="15min",
+            )
+        )
