@@ -1,15 +1,13 @@
 """
-Shap Explainer for RegressionModels
+Shap Explainer for SKLearnModels
 ------------------------------------
 A `shap explainer <https://github.com/slundberg/shap>`_ specifically for time series
 forecasting models.
 
-This class is (currently) limited to Darts' `RegressionModel` instances of forecasting models.
-It uses shap values to provide "explanations" of each input features.
-The input features are the different past lags (of the target and/or past covariates),
-as well as potential future lags of future covariates used as inputs by the forecasting
-model to produce its forecasts.
-Furthermore, in the case of multivariate series, the features contain each dimension of
+This class is (currently) limited to Darts' `SKLearnModel` instances of forecasting models. It uses shap values to
+provide "explanations" of each input features. The input features are the different past lags (of the target and/or
+past covariates), as well as potential future lags of future covariates used as inputs by the forecasting model to
+produce its forecasts. Furthermore, in the case of multivariate series, the features contain each dimension of
 each of the (lagged) series.
 
 .. note::
@@ -17,26 +15,30 @@ each of the (lagged) series.
    This means that it does not capture potential indirect influence that some lags
    may have on the target by influencing other lags.
 
+- :func:`explain() <ShapExplainer.explain>` generates the explanations for a given foreground series (or
+  background series, if foreground is not provided).
+- :func:`summary_plot() <ShapExplainer.summary_plot>` displays a shap plot summary for each horizon and each
+  component dimension of the target series.
+- :func:`force_plot_from_ts() <ShapExplainer.force_plot_from_ts>` displays a shap force_plot for one target
+  and one horizon, for a given target series. It displays shap values of each lag/covariate with an additive force
+   layout.
 """
 
+from collections.abc import Sequence
 from enum import Enum
-from typing import Dict, NewType, Optional, Sequence, Tuple, Union
+from typing import NewType, Optional, Union
 
 import matplotlib.pyplot as plt
 import pandas as pd
 import shap
-from numpy import integer
 from sklearn.multioutput import MultiOutputRegressor
 
 from darts import TimeSeries
-from darts.explainability.explainability import (
-    ExplainabilityResult,
-    ForecastingModelExplainer,
-)
+from darts.explainability.explainability import _ForecastingModelExplainer
+from darts.explainability.explainability_result import ShapExplainabilityResult
 from darts.logging import get_logger, raise_if, raise_log
-from darts.models.forecasting.regression_model import RegressionModel
-from darts.utils.data.tabularization import _create_lagged_data
-from darts.utils.utils import series2seq
+from darts.models.forecasting.sklearn_model import SKLearnModel
+from darts.utils.data.tabularization import create_lagged_prediction_data
 
 logger = get_logger(__name__)
 
@@ -58,10 +60,12 @@ class _ShapMethod(Enum):
 ShapMethod = NewType("ShapMethod", _ShapMethod)
 
 
-class ShapExplainer(ForecastingModelExplainer):
+class ShapExplainer(_ForecastingModelExplainer):
+    model: SKLearnModel
+
     def __init__(
         self,
-        model: RegressionModel,
+        model: SKLearnModel,
         background_series: Optional[Union[TimeSeries, Sequence[TimeSeries]]] = None,
         background_past_covariates: Optional[
             Union[TimeSeries, Sequence[TimeSeries]]
@@ -80,20 +84,18 @@ class ShapExplainer(ForecastingModelExplainer):
         - A background series is a `TimeSeries` used to train the shap explainer.
         - A foreground series is a `TimeSeries` that can be explained by a shap explainer after it has been fitted.
 
-        Currently, ShapExplainer only works with `RegressionModel` forecasting models.
+        Currently, `ShapExplainer` only works with `SKLearnModel` forecasting models.
         The number of explained horizons (t+1, t+2, ...) can be at most equal to `output_chunk_length` of `model`.
 
         Parameters
         ----------
         model
-            A `ForecastingModel` to be explained. It must be fitted first.
+            A `SKLearnModel` to be explained. It must be fitted first.
         background_series
-            One or several series to *train* the `ForecastingModelExplainer` along with any foreground series.
-            Consider using a reduced well-chosen backgroundto to reduce computation time.
-                - optional if `model` was fit on a single target series. By default,
-                  it is the `series` used at fitting time.
-
-                - mandatory if `model` was fit on multiple (list of) target series.
+            One or several series to *train* the `ShapExplainer` along with any foreground series.
+            Consider using a reduced well-chosen background to reduce computation time.
+            Optional if `model` was fit on a single target series. By default, it is the `series` used at fitting time.
+            Mandatory if `model` was fit on multiple (list of) target series.
         background_past_covariates
             A past covariates series or list of series that the model needs once fitted.
         background_future_covariates
@@ -113,13 +115,16 @@ class ShapExplainer(ForecastingModelExplainer):
             Optionally, additional keyword arguments passed to `shap_method`.
         Examples
         --------
+        >>> from darts.datasets import AirPassengersDataset
         >>> from darts.explainability.shap_explainer import ShapExplainer
         >>> from darts.models import LinearRegressionModel
         >>> series = AirPassengersDataset().load()
         >>> model = LinearRegressionModel(lags=12)
         >>> model.fit(series[:-36])
         >>> shap_explain = ShapExplainer(model)
+        >>> results = shap_explain.explain()
         >>> shap_explain.summary_plot()
+        >>> shap_explain.force_plot_from_ts()
         """
 
         # TODO
@@ -129,24 +134,39 @@ class ShapExplainer(ForecastingModelExplainer):
         # 2) a de-trend methodology for the target. It can be for
         # example target - moving_average(input_chunk_length).
 
-        if not issubclass(type(model), RegressionModel):
+        if not issubclass(type(model), SKLearnModel):
             raise_log(
                 ValueError(
-                    "Invalid `model` type. Currently, only models of type `RegressionModel` are supported."
+                    "Invalid `model` type. Currently, only models of type `SKLearnModel` are supported."
+                ),
+                logger,
+            )
+
+        if not model.multi_models:
+            raise_log(
+                ValueError(
+                    "Invalid `multi_models` value `False`. Currently, "
+                    "ShapExplainer only supports SKLearnModels "
+                    "with `multi_models=True`."
                 ),
                 logger,
             )
 
         super().__init__(
-            model,
-            background_series,
-            background_past_covariates,
-            background_future_covariates,
+            model=model,
+            background_series=background_series,
+            background_past_covariates=background_past_covariates,
+            background_future_covariates=background_future_covariates,
+            requires_background=True,
+            requires_covariates_encoding=True,
+            check_component_names=True,
+            test_stationarity=True,
         )
 
-        # As we only use RegressionModel, we fix the forecast n step ahead we want to explain as
-        # output_chunk_length
-        self.n = self.model.output_chunk_length
+        if model.supports_probabilistic_prediction:
+            logger.warning(
+                "The model is probabilistic, but num_samples=1 will be used for explainability."
+            )
 
         if shap_method is not None:
             shap_method = shap_method.upper()
@@ -187,38 +207,111 @@ class ShapExplainer(ForecastingModelExplainer):
         ] = None,
         horizons: Optional[Sequence[int]] = None,
         target_components: Optional[Sequence[str]] = None,
-    ) -> ExplainabilityResult:
+    ) -> ShapExplainabilityResult:
+        """
+        Explains a foreground time series and returns a :class:`ShapExplainabilityResult
+        <darts.explainability.explainability_result.ShapExplainabilityResult>`.
+        The results can be retrieved with method :func:`get_explanation()
+        <darts.explainability.explainability_result.ShapExplainabilityResult.get_explanation>`.
+        The result is a multivariate `TimeSeries` instance containing the 'explanation'
+        for the (horizon, target_component) forecast at any timestamp forecastable corresponding to
+        the foreground `TimeSeries` input.
+
+        The component name convention of this multivariate `TimeSeries` is:
+        ``"{name}_{type_of_cov}_lag_{idx}"``, where:
+
+        - ``{name}`` is the component name from the original foreground series (target, past, or future).
+        - ``{type_of_cov}`` is the covariates type. It can take 3 different values:
+          ``"target"``, ``"past_cov"`` or ``"future_cov"``.
+        - ``{idx}`` is the lag index.
+
+        Parameters
+        ----------
+        foreground_series
+            Optionally, one or a sequence of target `TimeSeries` to be explained. Can be multivariate.
+            If not provided, the background `TimeSeries` will be explained instead.
+        foreground_past_covariates
+            Optionally, one or a sequence of past covariates `TimeSeries` if required by the forecasting model.
+        foreground_future_covariates
+            Optionally, one or a sequence of future covariates `TimeSeries` if required by the forecasting model.
+        horizons
+            Optionally, an integer or sequence of integers representing the future time steps to be explained.
+            `1` corresponds to the first timestamp being forecasted.
+            All values must be `<=output_chunk_length` of the explained forecasting model.
+        target_components
+            Optionally, a string or sequence of strings with the target components to explain.
+
+        Returns
+        -------
+        ShapExplainabilityResult
+            The forecast explanations
+
+        Examples
+        --------
+        Say we have a model with 2 target components named ``"T_0"`` and ``"T_1"``,
+        3 past covariates with default component names ``"0"``, ``"1"``, and ``"2"``,
+        and one future covariate with default component name ``"0"``.
+        Also, ``horizons = [1, 2]``.
+        The model is a `SKLearnModel`, with ``lags = 3``, ``lags_past_covariates=[-1, -3]``,
+        ``lags_future_covariates = [0]``.
+
+        We provide `foreground_series`, `foreground_past_covariates`, `foreground_future_covariates` each of length 5.
+
+        >>> explain_results = explainer.explain(
+        >>>     foreground_series=foreground_series,
+        >>>     foreground_past_covariates=foreground_past_covariates,
+        >>>     foreground_future_covariates=foreground_future_covariates,
+        >>>     horizons=[1, 2],
+        >>>     target_names=["T_0", "T_1"])
+        >>> output = explain_results.get_explanation(horizon=1, component="T_1")
+        >>> feature_values = explain_results.get_feature_values(horizon=1, component="T_1")
+        >>> shap_objects = explain_results.get_shap_explanation_objects(horizon=1, component="T_1")
+
+        Then the method returns a multivariate TimeSeries containing the *explanations* of
+        the `ShapExplainer`, with the following component names:
+
+             - T_0_target_lag-1
+             - T_0_target_lag-2
+             - T_0_target_lag-3
+             - T_1_target_lag-1
+             - T_1_target_lag-2
+             - T_1_target_lag-3
+             - 0_past_cov_lag-1
+             - 0_past_cov_lag-3
+             - 1_past_cov_lag-1
+             - 1_past_cov_lag-3
+             - 2_past_cov_lag-1
+             - 2_past_cov_lag-3
+             - 0_fut_cov_lag_0
+
+        This series has length 3, as the model can explain 5-3+1 forecasts
+        (timestamp indexes 4, 5, and 6)
+        """
         super().explain(
             foreground_series, foreground_past_covariates, foreground_future_covariates
         )
-
-        if foreground_series is None:
-            foreground_series = self.background_series
-            foreground_past_covariates = self.background_past_covariates
-            foreground_future_covariates = self.background_future_covariates
-        else:
-            foreground_series = series2seq(foreground_series)
-            foreground_past_covariates = series2seq(foreground_past_covariates)
-            foreground_future_covariates = series2seq(foreground_future_covariates)
-
-            if self.model.encoders.encoding_available:
-                (
-                    foreground_past_covariates,
-                    foreground_future_covariates,
-                ) = self.model.generate_fit_encodings(
-                    series=foreground_series,
-                    past_covariates=foreground_past_covariates,
-                    future_covariates=foreground_future_covariates,
-                )
-
-        horizons, target_names = self._check_horizons_and_targets(
-            horizons, target_components
+        (
+            foreground_series,
+            foreground_past_covariates,
+            foreground_future_covariates,
+            _,
+            _,
+            _,
+            _,
+        ) = self._process_foreground(
+            foreground_series,
+            foreground_past_covariates,
+            foreground_future_covariates,
+        )
+        horizons, target_names = self._process_horizons_and_targets(
+            horizons,
+            target_components,
         )
 
         shap_values_list = []
-
+        feature_values_list = []
+        shap_explanation_object_list = []
         for idx, foreground_ts in enumerate(foreground_series):
-
             foreground_past_cov_ts = None
             foreground_future_cov_ts = None
 
@@ -240,31 +333,51 @@ class ShapExplainer(ForecastingModelExplainer):
             )
 
             shap_values_dict = {}
+            feature_values_dict = {}
+            shap_explanation_object_dict = {}
             for h in horizons:
-                tmp = {}
+                shap_values_dict_single_h = {}
+                feature_values_dict_single_h = {}
+                shap_explanation_object_dict_single_h = {}
                 for t in target_names:
-                    tmp[t] = TimeSeries.from_times_and_values(
-                        shap_[h][t].time_index,
-                        shap_[h][t].values,
-                        columns=shap_[h][t].feature_names,
+                    shap_values_dict_single_h[t] = TimeSeries(
+                        times=shap_[h][t].time_index,
+                        values=shap_[h][t].values,
+                        components=shap_[h][t].feature_names,
+                        copy=False,
                     )
-                shap_values_dict[h] = tmp
+                    feature_values_dict_single_h[t] = TimeSeries(
+                        times=shap_[h][t].time_index,
+                        values=shap_[h][t].data,
+                        components=shap_[h][t].feature_names,
+                        copy=False,
+                    )
+                    shap_explanation_object_dict_single_h[t] = shap_[h][t]
+                shap_values_dict[h] = shap_values_dict_single_h
+                feature_values_dict[h] = feature_values_dict_single_h
+                shap_explanation_object_dict[h] = shap_explanation_object_dict_single_h
 
             shap_values_list.append(shap_values_dict)
+            feature_values_list.append(feature_values_dict)
+            shap_explanation_object_list.append(shap_explanation_object_dict)
 
         if len(shap_values_list) == 1:
             shap_values_list = shap_values_list[0]
+            feature_values_list = feature_values_list[0]
+            shap_explanation_object_list = shap_explanation_object_list[0]
 
-        return ExplainabilityResult(shap_values_list)
+        return ShapExplainabilityResult(
+            shap_values_list, feature_values_list, shap_explanation_object_list
+        )
 
     def summary_plot(
         self,
-        horizons: Optional[Sequence[int]] = None,
-        target_components: Optional[Sequence[str]] = None,
+        horizons: Optional[Union[int, Sequence[int]]] = None,
+        target_components: Optional[Union[str, Sequence[str]]] = None,
         num_samples: Optional[int] = None,
         plot_type: Optional[str] = "dot",
         **kwargs,
-    ):
+    ) -> dict[int, dict[str, shap.Explanation]]:
         """
         Display a shap plot summary for each horizon and each component dimension of the target.
         This method reuses the initial background data as foreground (potentially sampled) to give a general importance
@@ -274,21 +387,25 @@ class ShapExplainer(ForecastingModelExplainer):
         Parameters
         ----------
         horizons
-            Optionally, a list of integers representing which points/steps in the future to explain,
-            starting from the first prediction step at 1. `horizons` must not be larger than
-            `output_chunk_length`.
+            Optionally, an integer or sequence of integers representing which points/steps in the future to explain,
+            starting from the first prediction step at 1. `horizons` must `<=output_chunk_length` of the forecasting
+            model.
         target_components
-            Optionally, a list of strings with the target components to be explained.
+            Optionally, a string or sequence of strings with the target components to explain.
         num_samples
-            Optionally, an integer for sampling the foreground series (based on the backgound),
+            Optionally, an integer for sampling the foreground series (based on the background),
             for the sake of performance.
         plot_type
-            Optionally, specify which of the propres shap library plot type to use. Can be one of
-            ``'dot', 'bar', 'violin'``.
+            Optionally, specify which of the shap library plot type to use. Can be one of ``'dot', 'bar', 'violin'``.
 
+        Returns
+        -------
+        shaps_
+            A nested dictionary {horizon : {component : shap.Explanation}} containing the raw Explanations for all
+            the horizons and components.
         """
 
-        horizons, target_components = self._check_horizons_and_targets(
+        horizons, target_components = self._process_horizons_and_targets(
             horizons, target_components
         )
 
@@ -305,17 +422,22 @@ class ShapExplainer(ForecastingModelExplainer):
 
         for t in target_components:
             for h in horizons:
-                plt.title("Target: `{}` - Horizon: {}".format(t, "t+" + str(h)))
+                plt.title(
+                    "Target: `{}` - Horizon: {}".format(
+                        t, "t+" + str(h + self.model.output_chunk_shift)
+                    )
+                )
                 shap.summary_plot(
                     shaps_[h][t],
                     foreground_X_sampled,
                     plot_type=plot_type,
                     **kwargs,
                 )
+        return shaps_
 
     def force_plot_from_ts(
         self,
-        foreground_series: TimeSeries = None,
+        foreground_series: Optional[TimeSeries] = None,
         foreground_past_covariates: Optional[TimeSeries] = None,
         foreground_future_covariates: Optional[TimeSeries] = None,
         horizon: Optional[int] = 1,
@@ -332,19 +454,19 @@ class ShapExplainer(ForecastingModelExplainer):
         Parameters
         ----------
         foreground_series
-            The target series to explain. Can be multivariate.
+            Optionally, the target series to explain. Can be multivariate. If `None`, will use the `background_series`.
         foreground_past_covariates
-            Optionally, a past covariate series if required by the forecasting model.
+            Optionally, a past covariate series if required by the forecasting model. If `None`, will use the
+            `background_past_covariates`.
         foreground_future_covariates
-            Optionally, a future covariate series if required by the forecasting model.
+            Optionally, a future covariate series if required by the forecasting model. If `None`, will use the
+            `background_future_covariates`.
         horizon
-            Optionally, an integer for the point/step in the future to explain,
-            starting from the first
-            prediction step at 1. `horizons` must not be larger than `output_chunk_length`.
-            by default, horizon = 1.
+            Optionally, an integer for the point/step in the future to explain, starting from the first prediction
+            step at 1. `horizons` must not be larger than `output_chunk_length`.
         target_component
-            Optionally, the target component to plot. If the target series is multivariate,
-            the target component must be specified.
+            Optionally, the target component to plot. If the target series is multivariate, the target component
+            must be specified.
         **kwargs
             Optionally, additional keyword arguments passed to `shap.force_plot()`.
         """
@@ -357,21 +479,24 @@ class ShapExplainer(ForecastingModelExplainer):
         if target_component is None:
             target_component = self.target_components[0]
 
-        horizon, target_component = self._check_horizons_and_targets(
-            horizon, target_component
+        (
+            foreground_series,
+            foreground_past_covariates,
+            foreground_future_covariates,
+            _,
+            _,
+            _,
+            _,
+        ) = self._process_foreground(
+            foreground_series,
+            foreground_past_covariates,
+            foreground_future_covariates,
         )
-
-        horizon, target_component = horizon[0], target_component[0]
-
-        if self.model.encoders.encoding_available:
-            (
-                foreground_past_covariates,
-                foreground_future_covariates,
-            ) = self.model.generate_fit_encodings(
-                series=foreground_series,
-                past_covariates=foreground_past_covariates,
-                future_covariates=foreground_future_covariates,
-            )
+        horizons, target_components = self._process_horizons_and_targets(
+            horizon,
+            target_component,
+        )
+        horizon, target_component = horizons[0], target_components[0]
 
         foreground_X = self.explainers._create_regression_model_shap_X(
             foreground_series, foreground_past_covariates, foreground_future_covariates
@@ -388,45 +513,13 @@ class ShapExplainer(ForecastingModelExplainer):
             **kwargs,
         )
 
-    def _check_horizons_and_targets(
-        self,
-        horizons: Union[int, Sequence[int]],
-        target_components: Union[str, Sequence[str]],
-    ) -> Tuple[Sequence[int], Sequence[str]]:
-
-        if target_components is not None:
-            if isinstance(target_components, str):
-                target_components = [target_components]
-            raise_if(
-                any(
-                    [
-                        target_name not in self.target_components
-                        for target_name in target_components
-                    ]
-                ),
-                "One of the target names doesn't exist. Please review your target_names input",
-            )
-        else:
-            target_components = self.target_components
-
-        if horizons is not None:
-            if isinstance(horizons, int):
-                horizons = [horizons]
-
-            raise_if(max(horizons) > self.n, "One of the horizons is too large.")
-            raise_if(min(horizons) < 1, "One of the horizons is too small.")
-        else:
-            horizons = range(1, self.n + 1)
-
-        return horizons, target_components
-
 
 class _RegressionShapExplainers:
     """
     Helper Class to wrap the different cases encountered with shap different explainers, multivariates,
     horizon etc.
-    Aim to provide shap values for any type of RegressionModel. Manage the MultioutputRegressor cases.
-    For darts RegressionModel only.
+    Aim to provide shap values for any type of SKLearnModel. Manage the MultioutputRegressor cases.
+    For darts SKLearnModel only.
     """
 
     default_sklearn_shap_explainers = {
@@ -435,15 +528,15 @@ class _RegressionShapExplainers:
         "CatBoostRegressor": _ShapMethod.TREE,
         "XGBRegressor": _ShapMethod.TREE,
         "GradientBoostingRegressor": _ShapMethod.TREE,
+        "HistGradientBoostingRegressor": _ShapMethod.TREE,
         # Tree models
         "DecisionTreeRegressor": _ShapMethod.TREE,
         "ExtraTreeRegressor": _ShapMethod.TREE,
+        "ExtraTreesRegressor": _ShapMethod.TREE,
+        "RandomForestRegressor": _ShapMethod.TREE,
         # Ensemble model
         "AdaBoostRegressor": _ShapMethod.PERMUTATION,
         "BaggingRegressor": _ShapMethod.PERMUTATION,
-        "ExtraTreesRegressor": _ShapMethod.PERMUTATION,
-        "HistGradientBoostingRegressor": _ShapMethod.PERMUTATION,
-        "RandomForestRegressor": _ShapMethod.PERMUTATION,
         "RidgeCV": _ShapMethod.PERMUTATION,
         "Ridge": _ShapMethod.PERMUTATION,
         # Linear models
@@ -474,7 +567,7 @@ class _RegressionShapExplainers:
 
     def __init__(
         self,
-        model: RegressionModel,
+        model: SKLearnModel,
         n: int,
         target_components: Sequence[str],
         past_covariates_components: Sequence[str],
@@ -482,11 +575,10 @@ class _RegressionShapExplainers:
         background_series: Sequence[TimeSeries],
         background_past_covariates: Sequence[TimeSeries],
         background_future_covariates: Sequence[TimeSeries],
-        shap_method: ShapMethod,
+        shap_method: _ShapMethod,
         background_num_samples: Optional[int] = None,
         **kwargs,
     ):
-
         self.model = model
         self.target_dim = self.model.input_dim["target"]
         self.is_multioutputregressor = isinstance(
@@ -521,7 +613,7 @@ class _RegressionShapExplainers:
                 self.explainers[i] = {}
                 for j in range(self.target_dim):
                     self.explainers[i][j] = self._build_explainer_sklearn(
-                        self.model.get_multioutput_estimator(horizon=i, target_dim=j),
+                        self.model.get_estimator(horizon=i, target_dim=j),
                         self.background_X,
                         self.shap_method,
                         **kwargs,
@@ -533,11 +625,10 @@ class _RegressionShapExplainers:
 
     def shap_explanations(
         self,
-        foreground_X,
+        foreground_X: pd.DataFrame,
         horizons: Optional[Sequence[int]] = None,
         target_components: Optional[Sequence[str]] = None,
-    ) -> Dict[integer, Dict[str, shap.Explanation]]:
-
+    ) -> dict[int, dict[str, shap.Explanation]]:
         """
         Return a dictionary of dictionaries of shap.Explanation instances:
         - the first dimension corresponds to the n forecasts ahead we want to explain (Horizon).
@@ -545,12 +636,12 @@ class _RegressionShapExplainers:
         Parameters
         ----------
         foreground_X
-            the Dataframe of lags features specific of darts RegressionModel.
+            the Dataframe of lags features specific of darts SKLearnModel.
         horizons
             Optionally, a list of integers representing which points/steps in the future we want to explain,
             starting from the first prediction step at 1. Currently, only forecasting models are supported which
             provide an `output_chunk_length` parameter. `horizons` must not be larger than `output_chunk_length`.
-        target_names
+        target_components
             Optionally, a list of strings with the target components we want to explain.
 
         """
@@ -559,10 +650,11 @@ class _RegressionShapExplainers:
         # native multiOutput estimators
         shap_explanations = {}
         if self.is_multioutputregressor:
-
             for h in horizons:
                 tmp_n = {}
-                for t_idx, t in enumerate(target_components):
+                for t_idx, t in enumerate(self.target_components):
+                    if t not in target_components:
+                        continue
                     explainer = self.explainers[h - 1][t_idx](foreground_X)
                     explainer.base_values = explainer.base_values.ravel()
                     explainer.time_index = foreground_X.index
@@ -573,13 +665,16 @@ class _RegressionShapExplainers:
             shap_explanation_tmp = self.explainers(foreground_X)
             for h in horizons:
                 tmp_n = {}
-                for t_idx, t in enumerate(target_components):
+                for t_idx, t in enumerate(self.target_components):
+                    if t not in target_components:
+                        continue
                     if not self.single_output:
                         tmp_t = shap.Explanation(
                             shap_explanation_tmp.values[
                                 :, :, self.target_dim * (h - 1) + t_idx
                             ]
                         )
+                        tmp_t.data = shap_explanation_tmp.data
                         tmp_t.base_values = shap_explanation_tmp.base_values[
                             :, self.target_dim * (h - 1) + t_idx
                         ].ravel()
@@ -601,7 +696,6 @@ class _RegressionShapExplainers:
         shap_method: Optional[ShapMethod] = None,
         **kwargs,
     ):
-
         model_name = type(model_sklearn).__name__
 
         # no shap methods - we need to take the default one
@@ -647,34 +741,40 @@ class _RegressionShapExplainers:
 
     def _create_regression_model_shap_X(
         self,
-        target_series,
-        past_covariates,
-        future_covariates,
-        n_samples=None,
-        train=False,
+        target_series: Optional[Union[TimeSeries, Sequence[TimeSeries]]],
+        past_covariates: Optional[Union[TimeSeries, Sequence[TimeSeries]]],
+        future_covariates: Optional[Union[TimeSeries, Sequence[TimeSeries]]],
+        n_samples: Optional[int] = None,
+        train: bool = False,
     ) -> pd.DataFrame:
         """
         Creates the shap format input for regression models.
         The output is a pandas DataFrame representing all lags of different covariates, and with adequate
         column names in order to map feature / shap values.
-        It uses create_lagged_data also used in RegressionModel to build the tabular dataset.
+        It uses create_lagged_data also used in SKLearnModel to build the tabular dataset.
 
         """
 
-        lags_list = self.model.lags.get("target")
-        lags_past_covariates_list = self.model.lags.get("past")
-        lags_future_covariates_list = self.model.lags.get("future")
+        lags_list = self.model._get_lags("target")
+        lags_past_covariates_list = self.model._get_lags("past")
+        lags_future_covariates_list = self.model._get_lags("future")
 
-        X, _, indexes = _create_lagged_data(
-            target_series,
-            self.n,
-            past_covariates,
-            future_covariates,
-            lags_list,
-            lags_past_covariates_list,
-            lags_future_covariates_list,
-            is_training=False,
+        X, indexes = create_lagged_prediction_data(
+            target_series=target_series if lags_list else None,
+            past_covariates=past_covariates if lags_past_covariates_list else None,
+            future_covariates=(
+                future_covariates if lags_future_covariates_list else None
+            ),
+            lags=lags_list,
+            lags_past_covariates=lags_past_covariates_list if past_covariates else None,
+            lags_future_covariates=(
+                lags_future_covariates_list if future_covariates else None
+            ),
+            uses_static_covariates=self.model.uses_static_covariates,
+            last_static_covariates_shape=self.model._static_covariates_shape,
         )
+        # Remove sample axis:
+        X = X[:, :, 0]
 
         if train:
             X = pd.DataFrame(X)
@@ -690,26 +790,11 @@ class _RegressionShapExplainers:
         if n_samples:
             X = shap.utils.sample(X, n_samples)
 
-        # We keep the creation order of the different lags/features in create_lagged_data
-        lags_names_list = []
-        if lags_list:
-            for lag in lags_list:
-                for t_name in self.target_components:
-                    lags_names_list.append(t_name + "_target_lag" + str(lag))
-        if lags_past_covariates_list:
-            for lag in lags_past_covariates_list:
-                for t_name in self.past_covariates_components:
-                    lags_names_list.append(t_name + "_past_cov_lag" + str(lag))
-        if lags_future_covariates_list:
-            for lag in lags_future_covariates_list:
-                for t_name in self.future_covariates_components:
-                    lags_names_list.append(t_name + "_fut_cov_lag" + str(lag))
-
+        # rename output columns to the matching lagged features names
         X = X.rename(
             columns={
-                name: lags_names_list[idx]
+                name: self.model.lagged_feature_names[idx]
                 for idx, name in enumerate(X.columns.to_list())
             }
         )
-
         return X
