@@ -9,8 +9,8 @@ import numpy as np
 from sklearn.gaussian_process import GaussianProcessRegressor
 from sklearn.gaussian_process.kernels import Kernel
 
+from darts import TimeSeries
 from darts.models.filtering.filtering_model import FilteringModel
-from darts.timeseries import TimeSeries
 
 
 class GaussianProcessFilter(FilteringModel):
@@ -73,4 +73,10 @@ class GaussianProcessFilter(FilteringModel):
 
         filtered_values = filtered_values.reshape(len(times), -1, num_samples)
 
-        return series.with_values(filtered_values)
+        return TimeSeries(
+            times=series.time_index,
+            values=filtered_values,
+            components=series.components,
+            copy=False,
+            **series._attrs,
+        )
