@@ -20,7 +20,7 @@ all copies or substantial portions of the Software.
 '
 """
 
-from typing import Dict, List, Optional, Tuple, Union
+from typing import Optional, Union
 
 import torch
 import torch.nn as nn
@@ -31,7 +31,7 @@ from darts.utils.torch import MonteCarloDropout
 
 logger = get_logger(__name__)
 
-HiddenState = Union[Tuple[torch.Tensor, torch.Tensor], torch.Tensor]
+HiddenState = Union[tuple[torch.Tensor, torch.Tensor], torch.Tensor]
 
 
 def get_embedding_size(n: int, max_size: int = 100) -> int:
@@ -55,7 +55,6 @@ class _TimeDistributedEmbeddingBag(nn.EmbeddingBag):
         self.batch_first = batch_first
 
     def forward(self, x):
-
         if len(x.size()) <= 2:
             return super().forward(x)
 
@@ -79,8 +78,8 @@ class _TimeDistributedEmbeddingBag(nn.EmbeddingBag):
 class _MultiEmbedding(nn.Module):
     def __init__(
         self,
-        embedding_sizes: Dict[str, Tuple[int, int]],
-        variable_names: List[str],
+        embedding_sizes: dict[str, tuple[int, int]],
+        variable_names: list[str],
     ):
         """Embedding layer for categorical variables including groups of categorical variables.
         Enabled for static and dynamic categories (i.e. 3 dimensions for batch x time x categories).
@@ -99,19 +98,19 @@ class _MultiEmbedding(nn.Module):
         self.embedding_sizes = embedding_sizes
         self.variable_names = variable_names
 
-        self.embeddings = nn.ModuleDict(
-            {name: nn.Embedding(*embedding_sizes[name]) for name in variable_names}
-        )
+        self.embeddings = nn.ModuleDict({
+            name: nn.Embedding(*embedding_sizes[name]) for name in variable_names
+        })
 
     @property
     def input_size(self) -> int:
         return len(self.variable_names)
 
     @property
-    def output_size(self) -> Union[Dict[str, int], int]:
+    def output_size(self) -> Union[dict[str, int], int]:
         return {name: sizes[1] for name, sizes in self.embedding_sizes.items()}
 
-    def forward(self, x: torch.Tensor) -> Dict[str, torch.Tensor]:
+    def forward(self, x: torch.Tensor) -> dict[str, torch.Tensor]:
         """
         Parameters
         ----------
@@ -151,7 +150,6 @@ class _TimeDistributedInterpolation(nn.Module):
         return upsampled
 
     def forward(self, x):
-
         if len(x.size()) <= 2:
             return self.interpolate(x)
 
@@ -383,13 +381,13 @@ class _GatedResidualNetwork(nn.Module):
 class _VariableSelectionNetwork(nn.Module):
     def __init__(
         self,
-        input_sizes: Dict[str, int],
+        input_sizes: dict[str, int],
         hidden_size: int,
-        input_embedding_flags: Optional[Dict[str, bool]] = None,
+        input_embedding_flags: Optional[dict[str, bool]] = None,
         dropout: float = 0.1,
         context_size: int = None,
-        single_variable_grns: Optional[Dict[str, _GatedResidualNetwork]] = None,
-        prescalers: Optional[Dict[str, nn.Linear]] = None,
+        single_variable_grns: Optional[dict[str, _GatedResidualNetwork]] = None,
+        prescalers: Optional[dict[str, nn.Linear]] = None,
         layer_norm: nn.Module = nn.LayerNorm,
     ):
         """
@@ -466,7 +464,7 @@ class _VariableSelectionNetwork(nn.Module):
     def num_inputs(self):
         return len(self.input_sizes)
 
-    def forward(self, x: Dict[str, torch.Tensor], context: torch.Tensor = None):
+    def forward(self, x: dict[str, torch.Tensor], context: torch.Tensor = None):
         if self.num_inputs > 1:
             # transform single variables
             var_outputs = []
@@ -543,12 +541,12 @@ class _InterpretableMultiHeadAttention(nn.Module):
         self.dropout = MonteCarloDropout(p=dropout)
 
         self.v_layer = nn.Linear(self.d_model, self.d_v)
-        self.q_layers = nn.ModuleList(
-            [nn.Linear(self.d_model, self.d_q) for _ in range(self.n_head)]
-        )
-        self.k_layers = nn.ModuleList(
-            [nn.Linear(self.d_model, self.d_k) for _ in range(self.n_head)]
-        )
+        self.q_layers = nn.ModuleList([
+            nn.Linear(self.d_model, self.d_q) for _ in range(self.n_head)
+        ])
+        self.k_layers = nn.ModuleList([
+            nn.Linear(self.d_model, self.d_k) for _ in range(self.n_head)
+        ])
         self.attention = _ScaledDotProductAttention()
         self.w_h = nn.Linear(self.d_v, self.d_model, bias=False)
 
@@ -561,7 +559,7 @@ class _InterpretableMultiHeadAttention(nn.Module):
             else:
                 torch.nn.init.zeros_(p)
 
-    def forward(self, q, k, v, mask=None) -> Tuple[torch.Tensor, torch.Tensor]:
+    def forward(self, q, k, v, mask=None) -> tuple[torch.Tensor, torch.Tensor]:
         heads = []
         attns = []
         vs = self.v_layer(v)
