@@ -4,14 +4,14 @@ Theta Method
 """
 
 import math
-from typing import List, Optional
+from typing import Optional
 
 import numpy as np
 import statsmodels.tsa.holtwinters as hw
 
+from darts import TimeSeries
 from darts.logging import get_logger, raise_if_not, raise_log
 from darts.models.forecasting.forecasting_model import LocalForecastingModel
-from darts.timeseries import TimeSeries
 from darts.utils.statistics import (
     check_seasonality,
     extract_trend_and_seasonality,
@@ -40,7 +40,7 @@ class Theta(LocalForecastingModel):
 
         `season_mode` must be a ``SeasonalityMode`` Enum member.
 
-        You can access the Enum with ``from darts import SeasonalityMode``.
+        You can access the Enum with ``from darts.utils.utils import SeasonalityMode``.
 
         Parameters
         ----------
@@ -58,6 +58,23 @@ class Theta(LocalForecastingModel):
         References
         ----------
         .. [1] `Unmasking the Theta method <https://robjhyndman.com/papers/Theta.pdf`
+
+        Examples
+        --------
+        >>> from darts.datasets import AirPassengersDataset
+        >>> from darts.models import Theta
+        >>> series = AirPassengersDataset().load()
+        >>> # using the canonical Theta method
+        >>> model = Theta(theta=2)
+        >>> model.fit(series)
+        >>> pred = model.predict(6)
+        >>> pred.values()
+        array([[442.7256909 ],
+               [433.74381763],
+               [494.54534585],
+               [480.36937856],
+               [481.06675142],
+               [545.80068173]])
         """
 
         super().__init__()
@@ -136,7 +153,12 @@ class Theta(LocalForecastingModel):
         return self
 
     def predict(
-        self, n: int, num_samples: int = 1, verbose: bool = False
+        self,
+        n: int,
+        num_samples: int = 1,
+        verbose: bool = False,
+        show_warnings: bool = True,
+        random_state: Optional[int] = None,
     ) -> "TimeSeries":
         super().predict(n, num_samples)
 
@@ -144,21 +166,17 @@ class Theta(LocalForecastingModel):
         forecast = self.model.forecast(n)
 
         # Forecast of the Linear Regression part.
-        drift = self.coef * np.array(
-            [
-                i + (1 - (1 - self.alpha) ** self.length) / self.alpha
-                for i in range(0, n)
-            ]
-        )
+        drift = self.coef * np.array([
+            i + (1 - (1 - self.alpha) ** self.length) / self.alpha for i in range(0, n)
+        ])
 
         # Combining the two forecasts
         forecast += drift
 
         # Re-apply the seasonal trend of the TimeSeries
         if self.is_seasonal:
-
             replicated_seasonality = np.tile(
-                self.seasonality.pd_series()[-self.season_period :],
+                self.seasonality.to_series()[-self.season_period :],
                 math.ceil(n / self.season_period),
             )[:n]
             if self.season_mode is SeasonalityMode.MULTIPLICATIVE:
@@ -168,8 +186,9 @@ class Theta(LocalForecastingModel):
 
         return self._build_forecast_series(forecast)
 
-    def __str__(self):
-        return f"Theta({self.theta})"
+    @property
+    def supports_multivariate(self) -> bool:
+        return False
 
     @property
     def min_train_series_length(self) -> int:
@@ -236,6 +255,22 @@ class FourTheta(LocalForecastingModel):
         -----
         Even though this model is an improvement of :class:`Theta`, it is a naive
         implementation of the algorithm, which can potentially be slower.
+
+        Examples
+        --------
+        >>> from darts.datasets import AirPassengersDataset
+        >>> from darts.models import FourTheta
+        >>> series = AirPassengersDataset().load()
+        >>> model = FourTheta(theta=2)
+        >>> model.fit(series)
+        >>> pred = model.predict(6)
+        >>> pred.values()
+        array([[443.3949283 ],
+               [434.39769555],
+               [495.28886231],
+               [481.08962991],
+               [481.78610361],
+               [546.61463773]])
         """
 
         super().__init__()
@@ -279,7 +314,7 @@ class FourTheta(LocalForecastingModel):
         self.length = len(series)
         # normalization of data
         if self.normalization:
-            self.mean = series.pd_dataframe(copy=False).mean().mean()
+            self.mean = series.to_dataframe(copy=False).mean().mean()
             raise_if_not(
                 not np.isclose(self.mean, 0),
                 "The mean value of the provided series is too close to zero to perform normalization",
@@ -366,7 +401,12 @@ class FourTheta(LocalForecastingModel):
         return self
 
     def predict(
-        self, n: int, num_samples: int = 1, verbose: bool = False
+        self,
+        n: int,
+        num_samples: int = 1,
+        verbose: bool = False,
+        show_warnings: bool = True,
+        random_state: Optional[int] = None,
     ) -> "TimeSeries":
         super().predict(n, num_samples)
 
@@ -385,9 +425,8 @@ class FourTheta(LocalForecastingModel):
 
         # Re-apply the seasonal trend of the TimeSeries
         if self.is_seasonal:
-
             replicated_seasonality = np.tile(
-                self.seasonality.pd_series()[-self.season_period :],
+                self.seasonality.to_series()[-self.season_period :],
                 math.ceil(n / self.season_period),
             )[:n]
             if self.season_mode is SeasonalityMode.MULTIPLICATIVE:
@@ -403,7 +442,7 @@ class FourTheta(LocalForecastingModel):
     @staticmethod
     def select_best_model(
         ts: TimeSeries,
-        thetas: Optional[List[int]] = None,
+        thetas: Optional[list[int]] = None,
         m: Optional[int] = None,
         normalization: bool = True,
         n_jobs: int = 1,
@@ -469,10 +508,9 @@ class FourTheta(LocalForecastingModel):
         )
         return theta
 
-    def __str__(self):
-        return "4Theta(theta:{}, curve:{}, model:{}, seasonality:{})".format(
-            self.theta, self.trend_mode, self.model_mode, self.season_mode
-        )
+    @property
+    def supports_multivariate(self) -> bool:
+        return False
 
     @property
     def min_train_series_length(self) -> int:
