@@ -1,12 +1,11 @@
-import unittest
-
+import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
+import pytest
 
+from darts import TimeSeries
 from darts.dataprocessing import dtw
 from darts.metrics import dtw_metric, mae, mape
-from darts.tests.base_test_class import DartsBaseTestClass
-from darts.timeseries import TimeSeries
 from darts.utils import timeseries_generation as tg
 
 
@@ -16,7 +15,7 @@ def _series_from_values(values):
     )
 
 
-class DynamicTimeWarpingTestCase(DartsBaseTestClass):
+class TestDynamicTimeWarping:
     length = 20
     freq = 1 / length
     series1 = tg.sine_timeseries(
@@ -62,17 +61,23 @@ class DynamicTimeWarpingTestCase(DartsBaseTestClass):
         series1 = _series_from_values(input1)
         series2 = _series_from_values(input2)
 
+        series1_copy = series1.copy()
+        series2_copy = series2.copy()
+
         exact_alignment = dtw.dtw(series1, series2, multi_grid_radius=-1)
 
-        self.assertEqual(
-            exact_alignment.distance(),
-            0,
-            "Minimum cost between two shifted series should be 0",
+        assert exact_alignment.distance() == 0, (
+            "Minimum cost between two shifted series should be 0"
         )
-        self.assertTrue(
-            np.array_equal(exact_alignment.path(), expected_path), "Incorrect path"
+        (
+            np.testing.assert_array_equal(exact_alignment.path(), expected_path),
+            "Incorrect path",
         )
 
+        # input series should not be modified
+        assert series1 == series1_copy
+        assert series2 == series2_copy
+
     def test_multi_grid(self):
         size = 2**5 - 1  # test odd size
         freq = 1 / size
@@ -83,11 +88,17 @@ class DynamicTimeWarpingTestCase(DartsBaseTestClass):
 
         series1 = _series_from_values(input1)
         series2 = _series_from_values(input2)
+        series1_copy = series1.copy()
+        series2_copy = series2.copy()
 
         exact_distance = dtw.dtw(series1, series2, multi_grid_radius=-1).distance()
         approx_distance = dtw.dtw(series1, series2, multi_grid_radius=1).distance()
 
-        self.assertAlmostEqual(exact_distance, approx_distance, 3)
+        assert round(abs(exact_distance - approx_distance), 3) == 0
+
+        # input series should not be modified
+        assert series1 == series1_copy
+        assert series2 == series2_copy
 
     def test_sakoe_chiba_window(self):
         window = 2
@@ -97,7 +108,7 @@ class DynamicTimeWarpingTestCase(DartsBaseTestClass):
         path = alignment.path()
 
         for i, j in path:
-            self.assertGreaterEqual(window, abs(i - j))
+            assert window >= abs(i - j)
 
     def test_itakura_window(self):
         n = 6
@@ -108,29 +119,26 @@ class DynamicTimeWarpingTestCase(DartsBaseTestClass):
         window.init_size(n, m)
 
         cells = list(window)
-        self.assertEqual(
-            cells,
-            [
-                (1, 1),
-                (1, 2),
-                (2, 1),
-                (2, 2),
-                (2, 3),
-                (3, 1),
-                (3, 2),
-                (3, 3),
-                (3, 4),
-                (4, 2),
-                (4, 3),
-                (4, 4),
-                (5, 2),
-                (5, 3),
-                (5, 4),
-                (5, 5),
-                (6, 4),
-                (6, 5),
-            ],
-        )
+        assert cells == [
+            (1, 1),
+            (1, 2),
+            (2, 1),
+            (2, 2),
+            (2, 3),
+            (3, 1),
+            (3, 2),
+            (3, 3),
+            (3, 4),
+            (4, 2),
+            (4, 3),
+            (4, 4),
+            (5, 2),
+            (5, 3),
+            (5, 4),
+            (5, 5),
+            (6, 4),
+            (6, 5),
+        ]
 
         sizes = [(10, 43), (543, 45), (34, 11)]
 
@@ -143,7 +151,7 @@ class DynamicTimeWarpingTestCase(DartsBaseTestClass):
             )
 
             dist = dtw.dtw(series1, series2, window=dtw.Itakura(slope)).mean_distance()
-            self.assertGreater(1, dist)
+            assert 1 > dist
 
     def test_warp(self):
         # Support different time dimension names
@@ -153,23 +161,29 @@ class DynamicTimeWarpingTestCase(DartsBaseTestClass):
         static_covs = pd.DataFrame([[0.0, 1.0]], columns=["st1", "st2"])
         series1 = TimeSeries.from_xarray(xa1).with_static_covariates(static_covs)
         series2 = TimeSeries.from_xarray(xa2).with_static_covariates(static_covs)
+        series1_copy = series1.copy()
+        series2_copy = series2.copy()
 
         alignment = dtw.dtw(series1, series2)
 
         warped1, warped2 = alignment.warped()
-        self.assertAlmostEqual(alignment.mean_distance(), mae(warped1, warped2))
+        assert round(abs(alignment.mean_distance() - mae(warped1, warped2)), 7) == 0
         assert warped1.static_covariates.equals(series1.static_covariates)
         assert warped2.static_covariates.equals(series2.static_covariates)
 
+        # input series should not be modified
+        assert series1 == series1_copy
+        assert series2 == series2_copy
+
         """
         See DTWAlignment.warped for why this functionality is currently disabled
 
         #Mutually Exclusive Option
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             alignment.warped(take_dates=True, range_index=True)
 
         #Take_dates does not support indexing by RangeIndex
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             xa3 = xa1.copy()
             xa3["time1"] = pd.RangeIndex(0, len(self.series1))
 
@@ -184,11 +198,11 @@ class DynamicTimeWarpingTestCase(DartsBaseTestClass):
         metric1 = dtw_metric(self.series1, self.series2, metric=mae)
         metric2 = dtw_metric(self.series1, self.series2, metric=mape)
 
-        self.assertGreater(0.5, metric1)
-        self.assertGreater(5, metric2)
+        assert 0.5 > metric1
+        assert 5 > metric2
 
     def test_nans(self):
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             series1 = _series_from_values([np.nan, 0, 1, 2, 3])
             series2 = _series_from_values([0, 1, 2, 3, 4])
 
@@ -198,6 +212,7 @@ class DynamicTimeWarpingTestCase(DartsBaseTestClass):
         align = dtw.dtw(self.series2, self.series1)
         align.plot()
         align.plot_alignment()
+        plt.close()
 
     def test_multivariate(self):
         n = 2
@@ -218,7 +233,7 @@ class DynamicTimeWarpingTestCase(DartsBaseTestClass):
             multi_series1, multi_series2, multi_grid_radius=radius
         )
 
-        self.assertTrue(np.all(alignment_uni.path() == alignment_multi.path()))
+        assert np.all(alignment_uni.path() == alignment_multi.path())
 
 
 # MINI_BENCHMARK
@@ -249,7 +264,3 @@ def _benchmark_dtw():
 
     cProfile.run("_dtw_exact()", sort="tottime")
     cProfile.run("_dtw_multigrid()", sort="tottime")
-
-
-if __name__ == "__main__":
-    unittest.main()
