@@ -2,17 +2,19 @@ from unittest.mock import Mock
 
 import numpy as np
 import pandas as pd
+import pytest
 
 from darts import TimeSeries
 from darts.logging import get_logger
 from darts.models import Prophet
-from darts.tests.base_test_class import DartsBaseTestClass
 from darts.utils import timeseries_generation as tg
+from darts.utils.utils import NotImportedModule, freqs, generate_index
 
 logger = get_logger(__name__)
 
 
-class ProphetTestCase(DartsBaseTestClass):
+@pytest.mark.skipif(isinstance(Prophet, NotImportedModule), reason="requires prophet")
+class TestProphet:
     def test_add_seasonality_calls(self):
         # test if adding seasonality at model creation and with method model.add_seasonality() are equal
         kwargs_mandatory = {
@@ -22,30 +24,37 @@ class ProphetTestCase(DartsBaseTestClass):
         }
         kwargs_mandatory2 = {
             "name": "custom2",
-            "seasonal_periods": 24,
+            "seasonal_periods": 24.9,
             "fourier_order": 1,
         }
-        kwargs_all = dict(kwargs_mandatory, **{"prior_scale": 1.0, "mode": "additive"})
+        kwargs_all = dict(
+            kwargs_mandatory,
+            **{
+                "prior_scale": 1.0,
+                "mode": "additive",
+                "condition_name": "custom_condition",
+            },
+        )
         model1 = Prophet(add_seasonalities=kwargs_all)
         model2 = Prophet()
         model2.add_seasonality(**kwargs_all)
-        self.assertEqual(model1._add_seasonalities, model2._add_seasonalities)
+        assert model1._add_seasonalities == model2._add_seasonalities
 
         # add multiple seasonalities
         model3 = Prophet(add_seasonalities=[kwargs_mandatory, kwargs_mandatory2])
-        self.assertEqual(len(model3._add_seasonalities), 2)
+        assert len(model3._add_seasonalities) == 2
 
         # seasonality already exists
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             model1.add_seasonality(**kwargs_mandatory)
 
         # missing mandatory arguments
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             for kw, arg in kwargs_mandatory.items():
                 Prophet(add_seasonalities={kw: arg})
 
         # invalid keywords
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             Prophet(
                 add_seasonalities=dict(
                     kwargs_mandatory, **{"some_random_keyword": "custom"}
@@ -53,10 +62,10 @@ class ProphetTestCase(DartsBaseTestClass):
             )
 
         # invalid value dtypes
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             Prophet(add_seasonalities=dict({kw: None for kw in kwargs_mandatory}))
 
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             Prophet(add_seasonalities=dict([]))
 
     def test_prophet_model(self):
@@ -64,24 +73,24 @@ class ProphetTestCase(DartsBaseTestClass):
         perform_full_test = False
 
         test_cases_all = {
-            "A": 12,
+            freqs["YE"]: 12,
             "W": 7,
-            "BM": 12,
+            freqs["BME"]: 12,
             "C": 5,
             "D": 7,
             "MS": 12,
             "B": 5,
-            "H": 24,
-            "BH": 8,
-            "Q": 4,
-            "min": 60,
-            "S": 60,
-            "30S": 60,
-            "24T": 60,
+            freqs["h"]: 24,
+            freqs["bh"]: 8,
+            freqs["QE"]: 4,
+            freqs["min"]: 60,
+            freqs["s"]: 60,
+            "30" + freqs["s"]: 60,
+            "24" + freqs["min"]: 60,
         }
 
         test_cases_fast = {
-            key: test_cases_all[key] for key in ["MS", "D", "H"]
+            key: test_cases_all[key] for key in ["MS", "D", freqs["h"]]
         }  # monthly, daily, hourly
 
         self.helper_test_freq_coversion(test_cases_all)
@@ -100,32 +109,34 @@ class ProphetTestCase(DartsBaseTestClass):
         model = Prophet(suppress_stdout_stderror=False)
         model._execute_and_suppress_output = Mock(return_value=True)
         model._model_builder = Mock(return_value=Mock(fit=Mock(return_value=True)))
-        df = pd.DataFrame(
-            {
-                "ds": pd.date_range(start="2022-01-01", periods=30, freq="D"),
-                "y": np.linspace(0, 10, 30),
-            }
-        )
+        df = pd.DataFrame({
+            "ds": pd.date_range(start="2022-01-01", periods=30, freq="D"),
+            "y": np.linspace(0, 10, 30),
+        })
         ts = TimeSeries.from_dataframe(df, time_col="ds", value_cols="y")
         model.fit(ts)
 
-        model._execute_and_suppress_output.assert_not_called(), "Suppression should not be called"
+        (
+            model._execute_and_suppress_output.assert_not_called(),
+            "Suppression should not be called",
+        )
         model.model.fit.assert_called_once(), "Model should still be fitted"
 
     def test_prophet_model_with_stdout_suppression(self):
         model = Prophet(suppress_stdout_stderror=True)
         model._execute_and_suppress_output = Mock(return_value=True)
         model._model_builder = Mock(return_value=Mock(fit=Mock(return_value=True)))
-        df = pd.DataFrame(
-            {
-                "ds": pd.date_range(start="2022-01-01", periods=30, freq="D"),
-                "y": np.linspace(0, 10, 30),
-            }
-        )
+        df = pd.DataFrame({
+            "ds": pd.date_range(start="2022-01-01", periods=30, freq="D"),
+            "y": np.linspace(0, 10, 30),
+        })
         ts = TimeSeries.from_dataframe(df, time_col="ds", value_cols="y")
         model.fit(ts)
 
-        model._execute_and_suppress_output.assert_called_once(), "Suppression should be called once"
+        (
+            model._execute_and_suppress_output.assert_called_once(),
+            "Suppression should be called once",
+        )
 
     def test_prophet_model_default_with_prophet_constructor(self):
         from prophet import Prophet as FBProphet
@@ -133,6 +144,27 @@ class ProphetTestCase(DartsBaseTestClass):
         model = Prophet()
         assert model._model_builder == FBProphet, "model should use Facebook Prophet"
 
+    def test_prophet_model_with_logistic_growth(self):
+        model = Prophet(growth="logistic", cap=1)
+
+        # Create timeseries with logistic function
+        times = generate_index(
+            pd.Timestamp("20200101"), pd.Timestamp("20210101"), freq="D"
+        )
+        values = np.linspace(-10, 10, len(times))
+        f = np.vectorize(lambda x: 1 / (1 + np.exp(-x)))
+        values = f(values)
+        ts = TimeSeries.from_times_and_values(times, values, freq="D")
+        # split in the middle, so the only way of predicting the plateau correctly
+        # is using the capacity
+        train, val = ts.split_after(0.5)
+
+        model.fit(train)
+        pred = model.predict(len(val))
+
+        for val_i, pred_i in zip(val.univariate_values(), pred.univariate_values()):
+            assert abs(val_i - pred_i) < 0.1
+
     def helper_test_freq_coversion(self, test_cases):
         for freq, period in test_cases.items():
             ts_sine = tg.sine_timeseries(
@@ -141,14 +173,16 @@ class ProphetTestCase(DartsBaseTestClass):
             # this should not raise an error if frequency is known
             _ = Prophet._freq_to_days(freq=ts_sine.freq_str)
 
-        self.assertAlmostEqual(
-            Prophet._freq_to_days(freq="30S"),
-            30 * Prophet._freq_to_days(freq="S"),
-            delta=10e-9,
+        assert (
+            abs(
+                Prophet._freq_to_days(freq="30" + freqs["s"])
+                - 30 * Prophet._freq_to_days(freq=freqs["s"])
+            )
+            < 10e-9
         )
 
         # check bad frequency string
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             _ = Prophet._freq_to_days(freq="30SS")
 
     def helper_test_prophet_model(self, period, freq, compare_all_models=False):
@@ -187,7 +221,7 @@ class ProphetTestCase(DartsBaseTestClass):
         model = Prophet(
             add_seasonalities=custom_seasonality,
             seasonality_mode="additive",
-            **supress_auto_seasonality
+            **supress_auto_seasonality,
         )
 
         model.fit(train, future_covariates=train_cov)
@@ -205,11 +239,62 @@ class ProphetTestCase(DartsBaseTestClass):
                 pred_raw_df[["ds", "yhat"]], time_col="ds"
             )
             compare_preds += [
-                pred_darts_stochastic.quantile_timeseries(0.5),
+                pred_darts_stochastic.quantile(0.5),
                 pred_raw,
             ]
 
         # all predictions should fit the underlying curve very well
         for pred in compare_preds:
             for val_i, pred_i in zip(val.univariate_values(), pred.univariate_values()):
-                self.assertAlmostEqual(val_i, pred_i, delta=0.1)
+                assert abs(val_i - pred_i) < 0.1
+
+    def test_conditional_seasonality(self):
+        """
+        Test that conditional seasonality is correctly incorporated by the model
+        """
+        duration = 395
+        horizon = 7
+        df = pd.DataFrame()
+        df["ds"] = pd.date_range(start="2022-01-02", periods=395)
+        df["y"] = [i + 10 * (i % 7 == 0) for i in range(duration)]
+        df["is_sunday"] = df["ds"].apply(lambda x: int(x.weekday() == 6))
+
+        ts = TimeSeries.from_dataframe(
+            df[:-horizon], time_col="ds", value_cols="y", freq="D"
+        )
+        future_covariates = TimeSeries.from_dataframe(
+            df, time_col="ds", value_cols=["is_sunday"], freq="D"
+        )
+        expected_result = TimeSeries.from_dataframe(
+            df[-horizon:], time_col="ds", value_cols="y", freq="D"
+        )
+
+        model = Prophet(seasonality_mode="additive")
+        model.add_seasonality(
+            name="weekly_sun",
+            seasonal_periods=7,
+            fourier_order=2,
+            condition_name="is_sunday",
+        )
+
+        model.fit(ts, future_covariates=future_covariates)
+
+        forecast = model.predict(horizon, future_covariates=future_covariates)
+
+        for val_i, pred_i in zip(
+            expected_result.univariate_values(), forecast.univariate_values()
+        ):
+            assert abs(val_i - pred_i) < 0.1
+
+        invalid_future_covariates = future_covariates.with_values(
+            np.reshape(np.random.randint(0, 3, duration), (-1, 1, 1)).astype("float")
+        )
+
+        with pytest.raises(ValueError):
+            model.fit(ts, future_covariates=invalid_future_covariates)
+
+        with pytest.raises(ValueError):
+            model.fit(
+                ts,
+                future_covariates=invalid_future_covariates.drop_columns("is_sunday"),
+            )
