@@ -4,24 +4,21 @@ Utils for Pytorch and its usage
 """
 
 from functools import wraps
-from inspect import signature
-from typing import Any, Callable, TypeVar
+from typing import Callable, TypeVar
 
+import numpy as np
 import torch.nn as nn
 import torch.nn.functional as F
-from numpy.random import randint
 from sklearn.utils import check_random_state
 from torch import Tensor
 from torch.random import fork_rng, manual_seed
 
-from darts.logging import get_logger, raise_if_not
+from darts.logging import get_logger, raise_log
+from darts.utils.utils import MAX_NUMPY_SEED_VALUE, MAX_TORCH_SEED_VALUE, _is_method
 
 T = TypeVar("T")
 logger = get_logger(__name__)
 
-MAX_TORCH_SEED_VALUE = (1 << 31) - 1  # to accommodate 32-bit architectures
-MAX_NUMPY_SEED_VALUE = (1 << 31) - 1
-
 
 class MonteCarloDropout(nn.Dropout):
     """
@@ -37,49 +34,20 @@ class MonteCarloDropout(nn.Dropout):
     often improves its performance.
     """
 
-    # We need to init it to False as some models may start by
-    # a validation round, in which case MC dropout is disabled.
-    mc_dropout_enabled: bool = False
-
-    def train(self, mode: bool = True):
-        # NOTE: we could use the line below if self.mc_dropout_rate represented
-        # a rate to be applied at inference time, and self.applied_rate the
-        # actual rate to be used in self.forward(). However, the original paper
-        # considers the same rate for training and inference; we also stick to this.
-
-        # self.applied_rate = self.p if mode else self.mc_dropout_rate
-
-        if mode:  # in train mode, keep dropout as is
-            self.mc_dropout_enabled = True
-        # in eval mode, bank on the mc_dropout_enabled flag
-        # mc_dropout_enabled is set equal to "mc_dropout" param given to predict()
+    # mc dropout is deactivated at init; see `MonteCarloDropout.mc_dropout_enabled` for more info
+    _mc_dropout_enabled = False
 
     def forward(self, input: Tensor) -> Tensor:
         # NOTE: we could use the following line in case a different rate
         # is used for inference:
         # return F.dropout(input, self.applied_rate, True, self.inplace)
-
         return F.dropout(input, self.p, self.mc_dropout_enabled, self.inplace)
 
-
-def _is_method(func: Callable[..., Any]) -> bool:
-    """Check if the specified function is a method.
-
-    Parameters
-    ----------
-    func
-        the function to inspect.
-
-    Returns
-    -------
-    bool
-        true if `func` is a method, false otherwise.
-    """
-    spec = signature(func)
-    if len(spec.parameters) > 0:
-        if list(spec.parameters.keys())[0] == "self":
-            return True
-    return False
+    @property
+    def mc_dropout_enabled(self) -> bool:
+        # mc dropout is only activated on `PLForecastingModule.on_predict_start()`
+        # otherwise, it is activated based on the `model.training` flag.
+        return self._mc_dropout_enabled or self.training
 
 
 def random_method(decorated: Callable[..., T]) -> Callable[..., T]:
@@ -91,24 +59,38 @@ def random_method(decorated: Callable[..., T]) -> Callable[..., T]:
     ----------
     decorated
         A method to be run in an isolated torch random context.
-
     """
     # check that @random_method has been applied to a method.
-    raise_if_not(
-        _is_method(decorated), "@random_method can only be used on methods.", logger
-    )
+    if not _is_method(decorated):
+        raise_log(ValueError("@random_method can only be used on methods."), logger)
 
     @wraps(decorated)
     def decorator(self, *args, **kwargs) -> T:
-        if "random_state" in kwargs.keys():
-            self._random_instance = check_random_state(kwargs["random_state"])
+        store_instance = False
+        random_instance = None
+        if "random_state" in kwargs.keys() and kwargs["random_state"] is not None:
+            # get random state from model constructor or `predict()`
+            random_instance = check_random_state(kwargs["random_state"])
+            if not hasattr(self, "_random_instance"):
+                # store random instance when called from model constructor
+                store_instance = True
         elif not hasattr(self, "_random_instance"):
-            self._random_instance = check_random_state(
-                randint(0, high=MAX_NUMPY_SEED_VALUE)
+            # get random state for first time from other method
+            store_instance = True
+            random_instance = check_random_state(
+                np.random.randint(0, high=MAX_NUMPY_SEED_VALUE)
             )
 
+        # if no random instance is provided, use the one stored in the class
+        if random_instance is None:
+            random_instance = self._random_instance
+
+        if store_instance:
+            self._random_instance = random_instance
+
+        # handle the randomness
         with fork_rng():
-            manual_seed(self._random_instance.randint(0, high=MAX_TORCH_SEED_VALUE))
+            manual_seed(random_instance.randint(0, high=MAX_TORCH_SEED_VALUE))
             return decorated(self, *args, **kwargs)
 
     return decorator
